/*  ArtNet Controller Plugin for XMMS
 *
 *  Copyright (C) 2004 Simon Newton
 *
 *  Portions copied from the Fine Spectrum Analyser Plugin,
 *   Copyright (C) 1998-2001 Vgvlgyi Attila, Peter Alm, Mikael Alm,
 *    Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config.h"

#include <xmms/plugin.h>
#include <xmms/util.h>

#include <sys/un.h>
#include <sys/socket.h>


/* WIDTH should be kept 256, this is the hardwired resolution of the
   spectrum given by XMMS */
#define WIDTH 256

/* number of CHANNELS */
#define CHANNELS 2

/* socket to write data to */
#define LOCAL_PATH "/tmp/artnet" 

static void artnet_playback_start(void);
static void artnet_playback_stop(void);
static void artnet_render_freq(gint16 data[2][WIDTH]);

static int sd ;

VisPlugin artnet_vp = {
	NULL,
	NULL,
	0,
	"ArtNet Controller Plugin " VERSION,
	0,
	CHANNELS,		
	NULL, /* init */
	NULL, /* cleanup */
	NULL, /* about */
	NULL, /* configure */
	NULL, /* disable_plugin */
	artnet_playback_start, /* playback_start */
	artnet_playback_stop, /* playback_stop */
	NULL, /* render_pcm */
	artnet_render_freq  /* render_freq */
};

VisPlugin *get_vplugin_info(void) {
	return &artnet_vp;
}

static void artnet_destroy_cb(GtkWidget *w,gpointer data) {
	artnet_vp.disable_plugin(&artnet_vp);
}


static void artnet_playback_start(void) {
	struct sockaddr_un servaddr ;
	int ret;

	// open socket
	sd = socket(AF_LOCAL, SOCK_STREAM, 0) ;
	
	if(! sd) {
		printf("socket failed\n") ;
		return  ;
	}
	
	memset(&servaddr, 0x00, sizeof(servaddr) );
	
	servaddr.sun_family = AF_LOCAL ;
	strcpy(servaddr.sun_path, LOCAL_PATH ) ;
	
	ret = connect(sd, (struct sockaddr *) &servaddr, sizeof(servaddr) ) ;

	if(ret) {
		printf("connect failed\n") ;
	}
}


static void artnet_playback_stop(void) {
	close(sd) ;
}


static void artnet_render_freq(gint16 data[CHANNELS][WIDTH]) {
	write(sd,data, 2*CHANNELS*WIDTH) ;
	return;
}
